@@ -58,11 +58,15 @@
 #include <common.h>
 #include <auth.h>
 #include <base64.h>
+#include <lease.h>
 
 extern int errno;
 
 struct prefix_ifconf *prefix_ifconflist;
 struct dhcp6_list siplist, sipnamelist, dnslist, dnsnamelist, ntplist;
+struct dhcp6_list nislist, nisnamelist;
+struct dhcp6_list nisplist, nispnamelist;
+struct dhcp6_list bcmcslist, bcmcsnamelist;
 long long optrefreshtime = -1;
 
 static struct dhcp6_ifconf *dhcp6_ifconflist;
@@ -71,7 +75,21 @@ static struct host_conf *host_conflist0, *host_conflist;
 static struct keyinfo *key_list, *key_list0;
 static struct authinfo *auth_list, *auth_list0;
 static struct dhcp6_list siplist0, sipnamelist0, dnslist0, dnsnamelist0, ntplist0;
+static struct dhcp6_list nislist0, nisnamelist0;
+static struct dhcp6_list nisplist0, nispnamelist0;
+static struct dhcp6_list bcmcslist0, bcmcsnamelist0;
 static long long optrefreshtime0;
+#ifndef DHCP6_DYNAMIC_HOSTCONF_MAX
+#define DHCP6_DYNAMIC_HOSTCONF_MAX	1024
+#endif
+struct dynamic_hostconf {
+	TAILQ_ENTRY(dynamic_hostconf) link;
+	struct host_conf *host;
+};
+static TAILQ_HEAD(dynamic_hostconf_listhead, dynamic_hostconf)
+	dynamic_hostconf_head;
+static unsigned int dynamic_hostconf_count;
+static struct pool_conf *pool_conflist, *pool_conflist0;
 
 enum { DHCPOPTCODE_SEND, DHCPOPTCODE_REQUEST, DHCPOPTCODE_ALLOW };
 
@@ -94,10 +112,14 @@ struct dhcp6_ifconf {
 
 	struct authinfo *authinfo; /* authentication information
 				    * (no need to clear) */
+	struct dhcp6_poolspec pool;
 };
 
 extern struct cf_list *cf_dns_list, *cf_dns_name_list, *cf_ntp_list;
 extern struct cf_list *cf_sip_list, *cf_sip_name_list;
+extern struct cf_list *cf_nis_list, *cf_nis_name_list;
+extern struct cf_list *cf_nisp_list, *cf_nisp_name_list;
+extern struct cf_list *cf_bcmcs_list, *cf_bcmcs_name_list;
 extern long long cf_refreshtime;
 extern char *configfilename;
 
@@ -113,8 +135,14 @@ static void clear_hostconf __P((struct host_conf *));
 static void clear_keys __P((struct keyinfo *));
 static void clear_authinfo __P((struct authinfo *));
 static int configure_duid __P((char *, struct duid *));
+static int configure_addr __P((struct cf_list *, struct dhcp6_list *, char *));
+static int configure_domain __P((struct cf_list *, struct dhcp6_list *, char *));
 static int get_default_ifid __P((struct prefix_ifconf *));
-static char *qstrdup __P((char *));
+static void clear_poolconf __P((struct pool_conf *));
+static struct pool_conf *create_pool __P((char *, struct dhcp6_range *));
+struct host_conf *find_dynamic_hostconf __P((struct duid *));
+static int in6_addr_cmp __P((struct in6_addr *, struct in6_addr *));
+static void in6_addr_inc __P((struct in6_addr *));
 
 int
 configure_interface(iflist)
@@ -233,6 +261,43 @@ configure_interface(iflist)
 				cp += strlen(ifc->scriptpath) - 1;
 				*cp = '\0'; /* clear the terminating quote */
 				break;
+			case DECL_ADDRESSPOOL:
+				{
+					struct dhcp6_poolspec* spec;
+					struct pool_conf* pool;
+
+					spec = (struct dhcp6_poolspec *)cfl->ptr;
+
+					for (pool = pool_conflist0; pool; pool = pool->next)
+						if (strcmp(spec->name, pool->name) == 0)
+							break;
+					if (pool == NULL) {
+						dprintf(LOG_ERR, FNAME, "%s:%d "
+							"pool '%s' not found",
+							configfilename, cfl->line,
+					   		spec->name);
+						goto bad;
+					}
+					if (spec->vltime != DHCP6_DURATION_INFINITE &&
+						(spec->pltime == DHCP6_DURATION_INFINITE ||
+						spec->pltime > spec->vltime)) {
+						dprintf(LOG_ERR, FNAME, "%s:%d ",
+							configfilename, cfl->line,
+							"specified a larger preferred lifetime "
+							"than valid lifetime");
+						goto bad;
+					}
+					ifc->pool = *spec;
+					if ((ifc->pool.name = strdup(spec->name)) == NULL) {
+						dprintf(LOG_ERR, FNAME,
+							"memory allocation failed");
+						goto bad;
+					}
+					dprintf(LOG_DEBUG, FNAME,
+						"pool '%s' is specified to the interface '%s'",
+						ifc->pool.name, ifc->ifname);
+				}
+				break;
 			default:
 				dprintf(LOG_ERR, FNAME, "%s:%d "
 					"invalid interface configuration",
@@ -534,6 +599,43 @@ configure_host(hostlist)
 				    "delayed auth with %s (keyid=%08x)",
 				    host->name, hconf->delayedkey->keyid);
 				break;
+			case DECL_ADDRESSPOOL:
+				{
+					struct dhcp6_poolspec* spec;
+					struct pool_conf *pool;
+
+					spec = (struct dhcp6_poolspec *)cfl->ptr;
+
+					for (pool = pool_conflist0; pool; pool = pool->next)
+						if (strcmp(spec->name, pool->name) == 0)
+							break;
+					if (pool == NULL) {
+						dprintf(LOG_ERR, FNAME, "%s:%d "
+							"pool '%s' not found",
+							configfilename, cfl->line,
+					   		spec->name);
+						goto bad;
+					}
+					if (spec->vltime != DHCP6_DURATION_INFINITE &&
+						(spec->pltime == DHCP6_DURATION_INFINITE ||
+						spec->pltime > spec->vltime)) {
+						dprintf(LOG_ERR, FNAME, "%s:%d ",
+							configfilename, cfl->line,
+							"specified a larger preferred lifetime "
+							"than valid lifetime");
+						goto bad;
+					}
+					hconf->pool = *spec;
+					if ((hconf->pool.name = strdup(spec->name)) == NULL) {
+						dprintf(LOG_ERR, FNAME,
+							"memory allocation failed");
+						goto bad;
+					}
+					dprintf(LOG_DEBUG, FNAME,
+						"pool '%s' is specified to the host '%s'",
+						hconf->pool.name, hconf->name);
+				}
+				break;
 			default:
 				dprintf(LOG_ERR, FNAME, "%s:%d "
 				    "invalid host configuration for %s",
@@ -874,166 +976,53 @@ configure_global_option()
 {
 	struct cf_list *cl;
 
-	/* check against configuration restriction */
-	if ((cf_dns_list && cf_dns_name_list) &&
-	    dhcp6_mode != DHCP6_MODE_SERVER) {
-		dprintf(LOG_INFO, FNAME, "%s:%d server-only configuration",
-		    configfilename, cf_dns_list->line);
-		goto bad;
-	}
-	if ((cf_sip_list && cf_sip_name_list) &&
-	    dhcp6_mode != DHCP6_MODE_SERVER) {
-		dprintf(LOG_INFO, FNAME, "%s:%d server-only configuration",
-		    configfilename, cf_sip_list->line);
-		goto bad;
-	}
-
 	/* SIP Server address */
-	TAILQ_INIT(&siplist0);
-	for (cl = cf_sip_list; cl; cl = cl->next) {
-		/* duplication check */
-		if (dhcp6_find_listval(&siplist0, DHCP6_LISTVAL_ADDR6,
-		    cl->ptr, 0)) {
-			dprintf(LOG_INFO, FNAME,
-			    "%s:%d duplicated SIP server: %s",
-			    configfilename, cl->line,
-			    in6addr2str((struct in6_addr *)cl->ptr, 0));
-			goto bad;
-		}
-		if (dhcp6_add_listval(&siplist0, DHCP6_LISTVAL_ADDR6,
-		    cl->ptr, NULL) == NULL) {
-			dprintf(LOG_ERR, FNAME, "failed to add a SIP server");
-			goto bad;
-		}
-	}
+	if (configure_addr(cf_sip_list, &siplist0, "SIP") < 0)
+		goto bad;
 
-	/* SIP Server domain name */
-	TAILQ_INIT(&sipnamelist0);
-	for (cl = cf_sip_name_list; cl; cl = cl->next) {
-		char *name, *cp;
-		struct dhcp6_vbuf name_vbuf;
-
-		name = strdup(cl->ptr + 1);
-		if (name == NULL) {
-			dprintf(LOG_ERR, FNAME, "failed to copy a SIP server domain name");
-			goto bad;
-		}
-		cp = name + strlen(name) - 1;
-		*cp = '\0';	/* clear the terminating quote */
-
-		name_vbuf.dv_buf = name;
-		name_vbuf.dv_len = strlen(name) + 1;
-
-		/* duplication check */
-		if (dhcp6_find_listval(&sipnamelist0, DHCP6_LISTVAL_VBUF,
-		    &name_vbuf, 0)) {
-			dprintf(LOG_INFO, FNAME,
-			    "%s:%d duplicated SIP name: %s",
-			    configfilename, cl->line, name_vbuf.dv_buf);
-			dhcp6_vbuf_free(&name_vbuf);
-			goto bad;
-		}
-
-		/* add the name */
-		if (dhcp6_add_listval(&sipnamelist0, DHCP6_LISTVAL_VBUF,
-		    &name_vbuf, NULL) == NULL) {
-			dprintf(LOG_ERR, FNAME, "failed to add a SIP name");
-			dhcp6_vbuf_free(&name_vbuf);
-			goto bad;
-		}
-		dhcp6_vbuf_free(&name_vbuf);
-	}
+	/* SIP domain name */
+	if (configure_domain(cf_sip_name_list, &sipnamelist0, "SIP") < 0)
+		goto bad;
 
 	/* DNS servers */
-	TAILQ_INIT(&dnslist0);
-	for (cl = cf_dns_list; cl; cl = cl->next) {
-		/* duplication check */
-		if (dhcp6_find_listval(&dnslist0, DHCP6_LISTVAL_ADDR6,
-		    cl->ptr, 0)) {
-			dprintf(LOG_INFO, FNAME,
-			    "%s:%d duplicated DNS server: %s",
-			    configfilename, cl->line,
-			    in6addr2str((struct in6_addr *)cl->ptr, 0));
-			goto bad;
-		}
-		if (dhcp6_add_listval(&dnslist0, DHCP6_LISTVAL_ADDR6,
-		    cl->ptr, NULL) == NULL) {
-			dprintf(LOG_ERR, FNAME, "failed to add a DNS server");
-			goto bad;
-		}
-	}
+	if (configure_addr(cf_dns_list, &dnslist0, "DNS") < 0)
+		goto bad;
 
 	/* DNS name */
-	TAILQ_INIT(&dnsnamelist0);
-	for (cl = cf_dns_name_list; cl; cl = cl->next) {
-		char *name, *cp;
-		struct dhcp6_vbuf name_vbuf;
-
-		name = strdup(cl->ptr + 1);
-		if (name == NULL) {
-			dprintf(LOG_ERR, FNAME, "failed to copy a DNS name");
-			goto bad;
-		}
-		cp = name + strlen(name) - 1;
-		*cp = '\0';	/* clear the terminating quote */
-
-		name_vbuf.dv_buf = name;
-		name_vbuf.dv_len = strlen(name) + 1;
-
-		/* duplication check */
-		if (dhcp6_find_listval(&dnsnamelist0, DHCP6_LISTVAL_VBUF,
-		    &name_vbuf, 0)) {
-			dprintf(LOG_INFO, FNAME,
-			    "%s:%d duplicated DNS name: %s",
-			    configfilename, cl->line, name_vbuf.dv_buf);
-			dhcp6_vbuf_free(&name_vbuf);
-			goto bad;
-		}
-
-		/* add the name */
-		if (dhcp6_add_listval(&dnsnamelist0, DHCP6_LISTVAL_VBUF,
-		    &name_vbuf, NULL) == NULL) {
-			dprintf(LOG_ERR, FNAME, "failed to add a DNS name");
-			dhcp6_vbuf_free(&name_vbuf);
-			goto bad;
-		}
-		dhcp6_vbuf_free(&name_vbuf);
-	}
+	if (configure_domain(cf_dns_name_list, &dnsnamelist0, "DNS") < 0)
+		goto bad;
 
 	/* NTP servers */
-	TAILQ_INIT(&ntplist0);
-	for (cl = cf_ntp_list; cl; cl = cl->next) {
-#ifdef USE_DH6OPT_NTP
-		/* duplication check */
-		if (dhcp6_find_listval(&ntplist0, DHCP6_LISTVAL_ADDR6,
-		    cl->ptr, 0)) {
-			dprintf(LOG_INFO, FNAME,
-			    "%s:%d duplicated NTP server: %s",
-			    configfilename, cl->line,
-			    in6addr2str((struct in6_addr *)cl->ptr, 0));
-			goto bad;
-		}
-		if (dhcp6_add_listval(&ntplist0, DHCP6_LISTVAL_ADDR6,
-		    cl->ptr, NULL) == NULL) {
-			dprintf(LOG_ERR, FNAME, "failed to add an NTP server");
-			goto bad;
-		}
-#else
-		dprintf(LOG_ERR, FNAME,
-		    "the support for NTP option is disabled");
+	if (configure_addr(cf_ntp_list, &ntplist0, "NTP") < 0)
+		goto bad;
+
+	/* NIS Server address */
+	if (configure_addr(cf_nis_list, &nislist0, "NIS") < 0)
+		goto bad;
+
+	/* NIS domain name */
+	if (configure_domain(cf_nis_name_list, &nisnamelist0, "NIS") < 0)
+		goto bad;
+
+	/* NIS+ Server address */
+	if (configure_addr(cf_nisp_list, &nisplist0, "NISP") < 0)
+		goto bad;
+
+	/* NIS+ domain name */
+	if (configure_domain(cf_nisp_name_list, &nispnamelist0, "NISP") < 0)
+		goto bad;
+
+	/* BCMCS Server address */
+	if (configure_addr(cf_bcmcs_list, &bcmcslist0, "BCMCS") < 0)
+		goto bad;
+
+	/* BCMCS domain name */
+	if (configure_domain(cf_bcmcs_name_list, &bcmcsnamelist0, "BCMCS") < 0)
 		goto bad;
-#endif
-	}
 
 	/* Lifetime for stateless options */
 	if (cf_refreshtime >= 0) {
-#ifdef USE_DH6OPT_REFRESHTIME
 		optrefreshtime0 = cf_refreshtime;
-#else
-		dprintf(LOG_ERR, FNAME, "the support for "
-		    "information refresh time option is disabled");
-		goto bad;
-#endif
 	}
 
 	return (0);
@@ -1042,6 +1031,102 @@ configure_global_option()
 	return (-1);		/* no need to free intermediate list */
 }
 
+static int
+configure_addr(cf_addr_list, list0, optname)
+	struct cf_list *cf_addr_list;
+	struct dhcp6_list *list0;
+	char *optname;
+{
+	struct cf_list *cl;
+
+	/* check against configuration restriction */
+	if (cf_addr_list != NULL && dhcp6_mode != DHCP6_MODE_SERVER) {
+		dprintf(LOG_INFO, FNAME, "%s:%d server-only configuration",
+		    configfilename, cf_addr_list->line);
+		return -1;
+	}
+
+	TAILQ_INIT(list0);
+	for (cl = cf_addr_list; cl; cl = cl->next) {
+		/* duplication check */
+		if (dhcp6_find_listval(list0, DHCP6_LISTVAL_ADDR6,
+		    cl->ptr, 0)) {
+			dprintf(LOG_INFO, FNAME,
+			    "%s:%d duplicated %s server: %s",
+			    configfilename, cl->line,
+			    optname,
+			    in6addr2str((struct in6_addr *)cl->ptr, 0));
+			return -1;
+		}
+		if (dhcp6_add_listval(list0, DHCP6_LISTVAL_ADDR6,
+		    cl->ptr, NULL) == NULL) {
+			dprintf(LOG_ERR, FNAME, "failed to add a %s server",
+			    optname);
+			return -1;
+		}
+	}
+
+	return 0;
+}
+
+static int
+configure_domain(cf_name_list, list0, optname)
+	struct cf_list *cf_name_list;
+	struct dhcp6_list *list0;
+	char *optname;
+{
+	struct cf_list *cl;
+
+	/* check against configuration restriction */
+	if (cf_name_list != NULL && dhcp6_mode != DHCP6_MODE_SERVER) {
+		dprintf(LOG_INFO, FNAME, "%s:%d server-only configuration",
+		    configfilename, cf_name_list->line);
+		return -1;
+	}
+
+	TAILQ_INIT(list0);
+	for (cl = cf_name_list; cl; cl = cl->next) {
+		char *name, *cp;
+		struct dhcp6_vbuf name_vbuf;
+
+		name = strdup(cl->ptr + 1);
+		if (name == NULL) {
+			dprintf(LOG_ERR, FNAME,
+			    "failed to copy a %s domain name",
+			    optname);
+			return -1;
+		}
+		cp = name + strlen(name) - 1;
+		*cp = '\0';	/* clear the terminating quote */
+
+		name_vbuf.dv_buf = name;
+		name_vbuf.dv_len = strlen(name) + 1;
+
+		/* duplication check */
+		if (dhcp6_find_listval(list0, DHCP6_LISTVAL_VBUF,
+		    &name_vbuf, 0)) {
+			dprintf(LOG_INFO, FNAME,
+			    "%s:%d duplicated %s name: %s",
+			    configfilename, cl->line, optname,
+			    name_vbuf.dv_buf);
+			dhcp6_vbuf_free(&name_vbuf);
+			return -1;
+		}
+
+		/* add the name */
+		if (dhcp6_add_listval(list0, DHCP6_LISTVAL_VBUF,
+		    &name_vbuf, NULL) == NULL) {
+			dprintf(LOG_ERR, FNAME, "failed to add a %s name",
+			    optname);
+			dhcp6_vbuf_free(&name_vbuf);
+			return -1;
+		}
+		dhcp6_vbuf_free(&name_vbuf);
+	}
+
+	return 0;
+}
+
 static int
 configure_duid(str, duid)
 	char *str;		/* this is a valid DUID string */
@@ -1210,6 +1295,7 @@ configure_cleanup()
 	dhcp6_clear_list(&ntplist0);
 	TAILQ_INIT(&ntplist0);
 	optrefreshtime0 = -1;
+	clear_poolconf(pool_conflist0);
 }
 
 void
@@ -1244,6 +1330,7 @@ configure_commit()
 		/* copy new configuration */
 		ifp->send_flags = ifc->send_flags;
 		ifp->allow_flags = ifc->allow_flags;
+		dhcp6_copy_list(&ifp->reqopt_list, &ifc->reqopt_list);
 		while ((iac = TAILQ_FIRST(&ifc->iaconf_list)) != NULL) {
 			TAILQ_REMOVE(&ifc->iaconf_list, iac, link);
 			TAILQ_INSERT_TAIL(&ifp->iaconf_list,
@@ -1258,6 +1345,8 @@ configure_commit()
 			ifp->authalgorithm = ifc->authinfo->algorithm;
 			ifp->authrdm = ifc->authinfo->rdm;
 		}
+		ifp->pool = ifc->pool;
+		ifc->pool.name = NULL;
 	}
 
 	clear_ifconf(dhcp6_ifconflist);
@@ -1289,7 +1378,7 @@ configure_commit()
 	dhcp6_clear_list(&siplist);
 	dhcp6_move_list(&siplist, &siplist0);
 
-	/* commit SIP server domain names */
+	/* commit SIP domain names */
 	dhcp6_clear_list(&sipnamelist);
 	dhcp6_move_list(&sipnamelist, &sipnamelist0);
 
@@ -1305,8 +1394,36 @@ configure_commit()
 	dhcp6_clear_list(&ntplist);
 	dhcp6_move_list(&ntplist, &ntplist0);
 
+	/* commit NIS server addresses */
+	dhcp6_clear_list(&nislist);
+	dhcp6_move_list(&nislist, &nislist0);
+
+	/* commit NIS domain names */
+	dhcp6_clear_list(&nisnamelist);
+	dhcp6_move_list(&nisnamelist, &nisnamelist0);
+
+	/* commit NIS+ server addresses */
+	dhcp6_clear_list(&nisplist);
+	dhcp6_move_list(&nisplist, &nisplist0);
+
+	/* commit NIS+ domain names */
+	dhcp6_clear_list(&nispnamelist);
+	dhcp6_move_list(&nispnamelist, &nispnamelist0);
+
+	/* commit BCMCS server addresses */
+	dhcp6_clear_list(&bcmcslist);
+	dhcp6_move_list(&bcmcslist, &bcmcslist0);
+
+	/* commit BCMCS domain names */
+	dhcp6_clear_list(&bcmcsnamelist);
+	dhcp6_move_list(&bcmcsnamelist, &bcmcsnamelist0);
+
 	/* commit information refresh time */
 	optrefreshtime = optrefreshtime0;
+	/* commit pool configuration */
+	clear_poolconf(pool_conflist);
+	pool_conflist = pool_conflist0;
+	pool_conflist0 = NULL;
 }
 
 static void
@@ -1326,6 +1443,8 @@ clear_ifconf(iflist)
 		if (ifc->scriptpath)
 			free(ifc->scriptpath);
 
+		if (ifc->pool.name)
+			free(ifc->pool.name);
 		free(ifc);
 	}
 }
@@ -1384,6 +1503,8 @@ clear_hostconf(hlist)
 		dhcp6_clear_list(&host->addr_list);
 		if (host->duid.duid_id)
 			free(host->duid.duid_id);
+		if (host->pool.name)
+			free(host->pool.name);
 		free(host);
 	}
 }
@@ -1538,6 +1659,12 @@ add_options(opcode, ifc, cfl0)
 		case DHCPOPT_DNS:
 		case DHCPOPT_DNSNAME:
 		case DHCPOPT_NTP:
+		case DHCPOPT_NIS:
+		case DHCPOPT_NISNAME:
+		case DHCPOPT_NISP:
+		case DHCPOPT_NISPNAME:
+		case DHCPOPT_BCMCS:
+		case DHCPOPT_BCMCSNAME:
 		case DHCPOPT_REFRESHTIME:
 			switch (cfl->type) {
 			case DHCPOPT_SIP:
@@ -1553,22 +1680,28 @@ add_options(opcode, ifc, cfl0)
 				opttype = DH6OPT_DNSNAME;
 				break;
 			case DHCPOPT_NTP:
-#ifdef USE_DH6OPT_NTP
 				opttype = DH6OPT_NTP;
-#else
-				dprintf(LOG_ERR, FNAME, "the support "
-				    "for NTP option is disabled");
-#endif
+				break;
+			case DHCPOPT_NIS:
+				opttype = DH6OPT_NIS_SERVERS;
+				break;
+			case DHCPOPT_NISNAME:
+				opttype = DH6OPT_NIS_DOMAIN_NAME;
+				break;
+			case DHCPOPT_NISP:
+				opttype = DH6OPT_NISP_SERVERS;
+				break;
+			case DHCPOPT_NISPNAME:
+				opttype = DH6OPT_NISP_DOMAIN_NAME;
+				break;
+			case DHCPOPT_BCMCS:
+				opttype = DH6OPT_BCMCS_SERVER_A;
+				break;
+			case DHCPOPT_BCMCSNAME:
+				opttype = DH6OPT_BCMCS_SERVER_D;
 				break;
 			case DHCPOPT_REFRESHTIME:
-#ifdef USE_DH6OPT_REFRESHTIME
 				opttype = DH6OPT_REFRESHTIME;
-#else
-				dprintf(LOG_ERR, FNAME, "the support "
-				    "for information refresh time option "
-				    "is disabled");
-				return (-1);
-#endif
 				break;
 			}
 			switch(opcode) {
@@ -1652,8 +1785,8 @@ add_prefix(head, name, type, prefix0)
 	}
 
 	/* validation about relationship of pltime and vltime */
-	if (oprefix.vltime != DHCP6_DURATITION_INFINITE &&
-	    (oprefix.pltime == DHCP6_DURATITION_INFINITE ||
+	if (oprefix.vltime != DHCP6_DURATION_INFINITE &&
+	    (oprefix.pltime == DHCP6_DURATION_INFINITE ||
 	    oprefix.pltime > oprefix.vltime)) {
 		if (type == DHCP6_LISTVAL_PREFIX6) {
 			dprintf(LOG_NOTICE, FNAME,
@@ -1699,6 +1832,10 @@ find_hostconf(duid)
 {
 	struct host_conf *host;
 
+	if ((host = find_dynamic_hostconf(duid)) != NULL) {
+		return (host);
+	}
+
 	for (host = host_conflist; host; host = host->next) {
 		if (host->duid.duid_len == duid->duid_len &&
 		    memcmp(host->duid.duid_id, duid->duid_id,
@@ -1760,7 +1897,7 @@ find_key(realm, realmlen, id)
 	return (NULL);
 }
 
-static char *
+char *
 qstrdup(qstr)
 	char *qstr;
 {
@@ -1779,3 +1916,341 @@ qstrdup(qstr)
 
 	return (dup);
 }
+
+int
+configure_pool(poollist)
+	struct cf_namelist *poollist;
+{
+	struct cf_namelist *plp;
+
+	dprintf(LOG_DEBUG, FNAME, "called");
+
+	if (poollist && dhcp6_mode != DHCP6_MODE_SERVER) {
+		dprintf(LOG_ERR, FNAME, "%s:%d "
+			"pool statement is server-only",
+			configfilename, poollist->line);
+		goto bad;
+	}
+
+	for (plp = poollist; plp; plp = plp->next) {
+		struct pool_conf *pool = NULL;
+		struct dhcp6_range *range = NULL;
+		struct cf_list *cfl;
+
+		for (cfl = plp->params; cfl; cfl = cfl->next) {
+			switch(cfl->type) {
+			case DECL_RANGE:
+				range = cfl->ptr;
+				break;
+			default:
+				dprintf(LOG_ERR, FNAME, "%s:%d "
+					"invalid pool configuration",
+					configfilename, cfl->line);
+				goto bad;
+			}
+		}
+
+		if (!range) {
+			dprintf(LOG_ERR, FNAME, "%s:%d "
+				"pool '%s' has no range declaration",
+				configfilename, plp->line,
+				plp->name);
+			goto bad;
+		}
+		if ((pool = create_pool(plp->name, range)) == NULL) {
+			dprintf(LOG_ERR, FNAME,
+				"faled to craete pool '%s'", plp->name);
+			goto bad;
+		}
+		pool->next = pool_conflist0;
+		pool_conflist0 = pool;
+	}
+
+	return (0);
+
+  bad:
+	/* there is currently nothing special to recover the error */
+	return (-1);
+}
+
+static void
+clear_poolconf(plist)
+	struct pool_conf *plist;
+{
+	struct pool_conf *pool, *pool_next;
+
+	dprintf(LOG_DEBUG, FNAME, "called");
+
+	for (pool = plist; pool; pool = pool_next) {
+		pool_next = pool->next;
+		free(pool->name);
+		free(pool);
+	}
+}
+
+struct host_conf *
+create_dynamic_hostconf(duid, pool)
+	struct duid *duid;
+	struct dhcp6_poolspec *pool;
+{
+	struct dynamic_hostconf *dynconf = NULL;
+	struct host_conf *host;
+	char* strid = NULL;
+	static int init = 1;
+
+	if (init) {
+		TAILQ_INIT(&dynamic_hostconf_head);
+		dynamic_hostconf_count = 0;
+		init = 0;
+	}
+
+	if (dynamic_hostconf_count >= DHCP6_DYNAMIC_HOSTCONF_MAX) {
+		struct dynamic_hostconf_listhead *head = &dynamic_hostconf_head;
+
+		dprintf(LOG_DEBUG, FNAME, "reached to the max count (count=%lu)",
+			dynamic_hostconf_count);
+
+		/* Find the last entry that doesn't need authentication */
+		TAILQ_FOREACH_REVERSE(dynconf, head, dynamic_hostconf_listhead, link)
+			if (dynconf->host->delayedkey == NULL)
+				break;
+		if (dynconf == NULL)
+			dynconf = TAILQ_LAST(head, dynamic_hostconf_listhead);
+		TAILQ_REMOVE(head, dynconf, link);
+		dynamic_hostconf_count--;
+		clear_hostconf(dynconf->host);
+	} else {
+		if ((dynconf = malloc(sizeof(*dynconf))) == NULL) {
+			dprintf(LOG_ERR, FNAME, "memory allocation failed");
+			return (NULL);
+		}
+	}
+	memset(dynconf, 0, sizeof(*dynconf));
+
+	if ((host = malloc(sizeof(*host))) == NULL) {
+		dprintf(LOG_ERR, FNAME, "memory allocation failed");
+		return (NULL);
+	}
+	memset(host, 0, sizeof(*host));
+	TAILQ_INIT(&host->prefix_list);
+	TAILQ_INIT(&host->addr_list);
+
+	if ((strid = duidstr(duid)) == NULL)
+		strid = "???";
+	if ((host->name = strdup(strid)) == NULL) {
+		dprintf(LOG_ERR, FNAME, "memory allocation failed");
+		goto bad;
+	}
+	if (duidcpy(&host->duid, duid) != 0) {
+		goto bad;
+	}
+	if (pool->name) {
+		if ((host->pool.name = strdup(pool->name)) == NULL) {
+			dprintf(LOG_ERR, FNAME, "memory allocation failed");
+			goto bad;
+		}
+	}
+	host->pool.pltime = pool->pltime;
+	host->pool.vltime = pool->vltime;
+
+	dynconf->host = host;
+	TAILQ_INSERT_HEAD(&dynamic_hostconf_head, dynconf, link);
+	dynamic_hostconf_count++; 
+
+	dprintf(LOG_DEBUG, FNAME, "created host_conf (name=%s)", host->name);
+
+	return (host);
+
+bad:
+	if (host)
+		clear_hostconf(host);	/* host->next must be NULL */
+	if (dynconf)
+		free(dynconf);
+
+	return (NULL);
+}
+
+struct host_conf *
+find_dynamic_hostconf(duid)
+	struct duid *duid;
+{
+	struct dynamic_hostconf *dynconf = NULL;
+
+	TAILQ_FOREACH(dynconf, &dynamic_hostconf_head, link) {
+		if (dynconf->host->duid.duid_len == duid->duid_len &&
+			memcmp(dynconf->host->duid.duid_id, duid->duid_id,
+			   duid->duid_len) == 0)
+			break;
+	}
+
+	if (dynconf) {
+		/* relocation */
+		TAILQ_REMOVE(&dynamic_hostconf_head, dynconf, link);
+		TAILQ_INSERT_HEAD(&dynamic_hostconf_head, dynconf, link);
+
+		return (dynconf->host);
+	}
+
+	return (NULL);
+}
+
+struct pool_conf *
+create_pool(name, range)
+	char *name;
+	struct dhcp6_range *range;
+{
+	struct pool_conf *pool = NULL;
+
+	if (!name || !range) {
+		return (NULL);
+	}
+
+	dprintf(LOG_DEBUG, FNAME, "name=%s, range=%s->%s", name,
+		in6addr2str(&range->min, 0), in6addr2str(&range->max, 0));
+
+	if (in6_addr_cmp(&range->min, &range->max) >= 0) {
+		dprintf(LOG_ERR, FNAME, "invalid address range %s->%s",
+			in6addr2str(&range->min, 0),
+			in6addr2str(&range->max, 0));
+		return (NULL);
+	}
+
+	if ((pool = malloc(sizeof(struct pool_conf))) == NULL) {
+		dprintf(LOG_ERR, FNAME, "memory allocation failed");
+		return (NULL);
+	}
+	if ((pool->name = strdup(name)) == NULL) {
+		dprintf(LOG_ERR, FNAME, "memory allocation failed");
+		return (NULL);
+	}
+	pool->cur = pool->min = range->min;
+	pool->max = range->max;
+
+	return (pool);
+}
+
+struct pool_conf *
+find_pool(name)
+	const char *name;
+{
+	struct pool_conf *pool = NULL;
+	
+	if (!name)
+		return (NULL);
+
+	dprintf(LOG_DEBUG, FNAME, "name=%s", name);
+
+	for (pool = pool_conflist; pool; pool = pool->next) {
+		if (strcmp(name, pool->name) == 0) {
+			dprintf(LOG_DEBUG, FNAME, "found (name=%s)", name);
+			return (pool);
+		}
+	}
+
+	dprintf(LOG_DEBUG, FNAME, "not found (name=%s)", name);
+
+	return (NULL);
+}
+
+int
+get_free_address_from_pool(pool, addr)
+	struct pool_conf *pool;
+	struct in6_addr *addr;
+{
+	int found = 0;
+	int round = 0;
+	struct in6_addr first;
+
+	if (!pool || !addr)
+		return (0);
+
+	dprintf(LOG_DEBUG, FNAME, "called (pool=%s)", pool->name);
+
+	memcpy(&first, &pool->cur, sizeof(first));
+
+	while (!found) {
+		if (!is_leased(&pool->cur) &&
+			!IN6_IS_ADDR_MULTICAST(&pool->cur) &&
+			!IN6_IS_ADDR_LINKLOCAL(&pool->cur) &&
+			!IN6_IS_ADDR_SITELOCAL(&pool->cur)) {
+			memcpy(addr, &pool->cur, sizeof(*addr));
+			found = 1;
+			dprintf(LOG_DEBUG, FNAME, "found %s",
+				in6addr2str(addr, 0));
+		}
+
+		if (in6_addr_cmp(&pool->cur, &pool->max) == 0) {
+			memcpy(&pool->next, &pool->min, sizeof(pool->next));
+			round = 1;
+		} else {
+			in6_addr_inc(&pool->cur);
+		}
+
+		dprintf(LOG_DEBUG, FNAME, "next address %s",
+			in6addr2str(&pool->cur, 0));
+
+		if (round && !found &&
+			in6_addr_cmp(&pool->cur, &first) == 0) {
+			dprintf(LOG_NOTICE, FNAME, "no available address");
+			break;
+		}
+	}
+
+	return (found);
+}
+
+int
+is_available_in_pool(pool, addr)
+	struct pool_conf *pool;
+	struct in6_addr *addr;
+{
+	if (!pool || !addr)
+		return (0);
+
+	dprintf(LOG_DEBUG, FNAME, "pool=%s, addr=%s",
+		 pool->name, in6addr2str(addr, 0));
+
+	if (in6_addr_cmp(addr, &pool->min) >= 0 &&
+		in6_addr_cmp(addr, &pool->max) <= 0 &&
+		!is_leased(addr) &&
+		!IN6_IS_ADDR_MULTICAST(addr) &&
+		!IN6_IS_ADDR_LINKLOCAL(addr) &&
+		!IN6_IS_ADDR_SITELOCAL(addr)) {
+		return (1);
+	}
+
+	dprintf(LOG_DEBUG, FNAME, "unavailable address (pool=%s, addr=%s)",
+		 pool->name, in6addr2str(addr, 0));
+
+	return (0);
+}
+
+static int 
+in6_addr_cmp(addr1, addr2)
+	struct in6_addr *addr1, *addr2;
+{
+	int i;
+
+	for (i = 0; i < 16; i++) {
+		if (addr1->s6_addr[i] != addr2->s6_addr[i]) {
+        	if (addr1->s6_addr[i] > addr2->s6_addr[i])
+				return (1);
+			else
+				return (-1);
+		}
+	}
+
+	return (0);
+}
+
+static void
+in6_addr_inc(addr)
+	struct in6_addr *addr;
+{
+	int i;
+
+	for (i = 15; i >= 0; i--) {
+		if (++(addr->s6_addr[i]) != 0x00)
+			break;
+	}
+}
